﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SUBD.ComponentInterfaces;

namespace SUBD.ComponentEntity
{
    public class ComponentLogic : IComponentLogic
	{
		private readonly IComponentStorage _componentStorage;
		public ComponentLogic(IComponentStorage componentStorage)
		{
			_componentStorage = componentStorage;
		}
		public List<ComponentViewModel>? ReadList(ComponentSearchModel? model)
		{
			var list = model == null ? _componentStorage.GetFullList() : _componentStorage.GetFilteredList(model);
			if (list == null)
			{
				return null;
			}
			return list;
		}
		public ComponentViewModel? ReadElement(ComponentSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			var element = _componentStorage.GetElement(model);
			if (element == null)
			{
				return null;
			}
			return element;
		}
		public bool Create(ComponentBindingModel model)
		{
			CheckModel(model);
			if (_componentStorage.Insert(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Update(ComponentBindingModel model)
		{
			CheckModel(model);
			if (_componentStorage.Update(model) == null)
			{
				return false;
			}
			return true;
		}
		public bool Delete(ComponentBindingModel model)
		{
			CheckModel(model, false);
			if (_componentStorage.Delete(model) == null)
			{
				return false;
			}
			return true;
		}
		private void CheckModel(ComponentBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.ComponentName))
			{
				throw new ArgumentNullException("Нет названия компонента",
			   nameof(model.ComponentName));
			}
			if (model.Cost <= 0)
			{
				throw new ArgumentNullException("Цена компонента должна быть больше 0", nameof(model.Cost));
			}
			var element = _componentStorage.GetElement(new ComponentSearchModel
			{
				ComponentName = model.ComponentName
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Компонент с таким названием уже есть");
			}
		}
	}
}
